package edu.villanova.ftp;

import java.net.Socket;
import java.net.ServerSocket;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.PrintStream;
import java.io.FileOutputStream;
import java.util.Date;
import java.util.TreeMap;
import java.util.Properties;
import edu.villanova.ftp.command.CommandList;
import edu.villanova.ftp.command.CommandListFactory;
import edu.villanova.ftp.security.ISecurityContext;
import edu.villanova.ftp.security.ISecurityContextFactory;
import edu.villanova.ftp.util.Loader;
import edu.villanova.ftp.util.Logger;

public class Ftp {

public static void main(String[] args) {

    if(args.length!=1) {
      System.out.println("Usage: java edu.villanova.Ftp <config file>");
      return;
    }

    CommandList commands = new CommandList();
    Properties prop = new Properties();
    Logger logger = null;
    ISecurityContext context = null;
    String log = null;
    String errlog = null;
    int port = 21;

    try {
      prop.load(new FileInputStream(args[0]));
    } catch (Exception e) {
      System.out.println("An Error occurred while loading the configuration file check error file for details.");
      System.err.println(new Date());
      e.printStackTrace();
      return;
    }

    try {
      log = prop.getProperty("Log");
      File t = new File(log);
      if((t.exists())) {
        t.renameTo(new File(log + "." + new Date().getTime()));
      } else {
        t.createNewFile();
      }
      logger = new Logger(new PrintStream(new FileOutputStream(t),true));
      logger.log("Server Started");
    } catch(Exception e) {
      e.printStackTrace();
      return;
    }

    try {
      errlog = prop.getProperty("ErrorLog");
      File t = new File(errlog);
      if((t.exists())) {
        t.renameTo(new File(errlog + "." + new Date().getTime()));
      } else {
        t.createNewFile();
      }
      System.setErr(new PrintStream(new FileOutputStream(t),true));
    } catch(Exception e) {
      e.printStackTrace();
      return;
    }

    try {
      context = ((ISecurityContextFactory)Loader.load(prop.getProperty("SecurityContextFactory"))).loadSecurityContext(prop);
    } catch (Exception e) {
      System.out.println("An Error occurred while loading the SecurityContext check error file for details.");
      logger.log("An Error occurred while loading the SecurityContext check error file for details.");
      logger.log("Server Stoped");
      System.err.println(new Date());
      e.printStackTrace();
      return;
    }

    try {
      commands = CommandListFactory.loadCommandList(prop.getProperty("CommandsFile"));
    } catch (Exception e) {
      System.out.println("An Error occurred while loading the CommandList check error file for details.");
      logger.log("An Error occurred while loading the CommandList check error file for details.");
      logger.log("Server Stoped");
      System.err.println(new Date());
      e.printStackTrace();
      return;
    }

    try {
      port = Integer.parseInt(prop.getProperty("Port"));
    } catch(Exception e) {
      System.err.println(new Date());
      e.printStackTrace();
    }

    try {
      ServerSocket s = new ServerSocket(port);
      for(;;) {
        Socket incoming = s.accept();
        new FtpServer(incoming,prop,commands,logger,context).start();
      } // for
    } catch(Exception e) {
      logger.log("Server Stopped check error log file.");
      System.out.println(new Date());
      e.printStackTrace();
    } // catch
  } // main
} // Ftp
