package edu.villanova.ftp;

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.Properties;
import java.util.Date;
import edu.villanova.ftp.command.CommandList;
import edu.villanova.ftp.security.ISecurityContext;
import edu.villanova.ftp.util.Logger;
import edu.villanova.ftp.command.ICommand;
import edu.villanova.ftp.security.IFtpAccount;

public class FtpServer extends Thread {

private Socket incoming;
private Properties props;
private CommandList commands;
private Logger logger;
private ISecurityContext context;

  public FtpServer(Socket incoming, Properties props, CommandList commands, Logger logger, ISecurityContext context) {
    this.incoming = incoming;
    this.props = props;
    this.commands = commands;
    this.logger = logger;
    this.context = context;
  }

  public void run() {
    BufferedReader in = null;
    IFtpAccount user = null;
    try {
      user = context.initializeFtpAccount(incoming, logger);
      in = new BufferedReader(new InputStreamReader(incoming.getInputStream()));
      user.getOut().println("220 Ftp server ready.\r");
      while(!user.isDone()) {
        String input = in.readLine();
        int range = input.indexOf(" ");
        if(range == -1)
          range = input.length();
        String command = input.substring(0,range).toUpperCase();
        if(commands.hasCommand(command)) {
          user.setCommand(input);
          try {
            ICommand com = (ICommand)commands.getCommand(command);
            if(context.commandAutherization(user,com)) {
              if(!com.run(user)) {
                user.getOut().println("451 Server Error");
              }
            } else {
              user.getOut().println("550 " + input + ": Permission denied");
            }
          } catch (Exception e) {
            user.getOut().println("451 Server Error");
            System.err.println(new Date() + ":" + input + ":" + user.getHost() + ":" + user.getUID());
            e.printStackTrace();
          }
        } else {
          if(command.equals("PASS")) {
            String pass = input.substring(4);
            pass = pass.trim();
            if(context.authenticate(user,pass)) {
              logger.log("Logged in",user);
              user.getOut().println("230 User "+user.getUID()+" logged in.");
            } else {
              logger.log("LOG IN FAILED!",user);
              user.isDone(true);
            }
          } else {
            logger.log("Not implemented :" + input, user);
            user.getOut().println("202 Not implemented");
          }
        }
        if(user.getCommand()!=null) {
          user.setLastCommand(user.getCommand());
        }
        user.setCommand("");
      }
    } catch (Exception e) {
      logger.log("Abnormal termination of server see error logs",user);
      System.err.println(new Date());
      e.printStackTrace();
    } finally {
      try {
        incoming.close();
      } catch(Exception e) {
        System.err.println(new Date());
        e.printStackTrace();
      }
      try {
        in.close();
      } catch(Exception e) {
        System.err.println(new Date());
        e.printStackTrace();
      }
    }
  }
}
